#!/bin/sh

# Copyright 2003-2004 The Apache Software Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Author: Samisa Abeysinghe (sabeysinghe@virtusa.com)


# *** Set CVS related variables ***
CVSROOT=":pserver:anoncvs:anoncvs@cvs.apache.org:/home/cvspublic"
CVS_RSH=/usr/bin/ssh
HOME_DIR=$PWD
CHECKOUT_DIR=cvsAutoBuild
LOG=${HOME_DIR}/log
ERROR_LOG=${HOME_DIR}/log_error
SOURCE_BUILD_MESSAGES=${HOME_DIR}/log_source_build_messages_`date +%Y-%m-%d@%H:%M:%S`
SOURCE_BUILD_ERRORS=${HOME_DIR}/log_source_build_errors_`date +%Y-%m-%d@%H:%M:%S`
SAMPLES_BUILD_MESSAGES=${HOME_DIR}/log_samples_build_messages_`date +%Y-%m-%d@%H:%M:%S`
SAMPLES_BUILD_ERRORS=${HOME_DIR}/log_samples_build_errors_`date +%Y-%m-%d@%H:%M:%S`
SAMPLES_RUN_MESSAGES=${HOME_DIR}/log_samples_run_messages_`date +%Y-%m-%d@%H:%M:%S`
SAMPLES_RUN_ERRORS=${HOME_DIR}/log_samples_run_errors_`date +%Y-%m-%d@%H:%M:%S`
export CVSROOT CVS_RSH HOME_DIR CHECKOUT_DIR LOG ERROR_LOG SOURCE_BUILD_MESSAGES SOURCE_BUILD_ERRORS SAMPLES_BUILD_MESSAGES SAMPLES_BUILD_ERRORS SAMPLES_RUN_MESSAGES SAMPLES_RUN_ERRORS

#get command line options
GET_CVS="true"
USE_APACHE="2"
while getopts n:a:h o
do	case "$o" in
	n)	GET_CVS="false";;
	a)	USE_APACHE=$OPTARG;;
	h)	echo "Usage: $0 [-a1|-a2] [-nocvc] [-help]"
		echo "Apache version:"
		echo "	-a1  	Use Apache 1.3 for deployement"
		echo "	-a2 	Use Apache 2.0 for deployement"
		echo "Chekout from cvs:"
		echo "	-nocvc	Do not chekout from cvs; Default is to checkout"				
		exit 1;;
	esac
done
#if [ -a ${CHECKOUT_DIR} ] 
#then
#	rm -rf ${CHECKOUT_DIR}
#fi

# *** Get the update from CVS ***
echo Getting CVS Update from ${CVSROOT}
cd ${HOME_DIR}

if [ ${GET_CVS} = "true" ]
then
	cvs checkout -d ${CHECKOUT_DIR} ws-axis/c
fi

if [ $? = 0 ]
then
	echo CVS Update  Sucessfull 
	echo `date` CVS Update  Sucessfull >> ${LOG}
else
	echo CVS Update Failed 
	echo `date` CVS Update Failed >> ${LOG}
	exit
fi

# *** Set build related variables ***
# if Apache 2.0
APACHE_HOME=/usr/local/apache2
APACHE_VERSION=apache2_0
# if Apache 1.3
#APACHE_HOME=/usr/local/apache
#APACHE_VERSION=apache1_3

XERCES_HOME=/home/samisa/cpp/xerces-c-src2_2_0
EXPAT_HOME=/home/samisa/expat-1.95.7
AXISCPP_HOME=${HOME_DIR}/${CHECKOUT_DIR}
export APACHE_HOME XERCES_HOME EXPAT_HOME AXISCPP_HOME 

# *** Copy headers (Apache and Xerces) ***
cp ${APACHE_HOME}/include/* ${AXISCPP_HOME}/include/${APACHE_VERSION}
cp -r ${XERCES_HOME}/include/xercesc/* ${AXISCPP_HOME}/include/xercesc
cp ${EXPAT_HOME}/lib/expat.h ${AXISCPP_HOME}/include/expat

# *** Copy configure and make scripts for Xerces
cp configure.ac ${AXISCPP_HOME}
cp src.server.Makefile.am ${AXISCPP_HOME}/src/server/Makefile.am
#cp ${AXISCPP_HOME}/src/soap/Makefile.am_xercesc ${AXISCPP_HOME}/src/soap/Makefile.am
#cp ${AXISCPP_HOME}/src/wsdd/Makefile.am_xercesc ${AXISCPP_HOME}/src/wsdd/Makefile.am

# *** Build Source ***
echo Starting Source Build
cd ${AXISCPP_HOME}
echo Build messages of build @ `date` > ${SOURCE_BUILD_MESSAGES}
echo Build errors/warnings of build @ `date` > ${SOURCE_BUILD_ERRORS}

sh build.sh >> ${SOURCE_BUILD_MESSAGES} 2>>${SOURCE_BUILD_ERRORS}

if [ $? = 0 ]
then
	echo Source Build  Sucessfull
	echo `date` Source Build  Sucessfull >> ${LOG}
else
	echo Source Build Failed
	echo `date` Source Build Failed >> ${LOG}
	exit
fi

echo See the following log files for details
echo "	Summary log 			:${LOG}"
echo "	Build messages 			:${SOURCE_BUILD_MESSAGES}"
echo "	Build errors/warnings 		:${SOURCE_BUILD_ERRORS}"



# *** Prepare to deploy with Apache 2 ***
echo Start preparing to deploy with ${APACHE_VERSION} ${USE_APACHE}
if [ ${USE_APACHE} = "2" ]
then
	cp -r ${APACHE_HOME}/modules ${HOME_DIR}/${APACHE_VERSION}
elif [ ${USE_APACHE} = "1" ]
then
	cp -r ${APACHE_HOME}/libexec ${HOME_DIR}/${APACHE_VERSION}
fi

cp -r ${APACHE_HOME}/logs ${HOME_DIR}/${APACHE_VERSION}
cp -r ${APACHE_HOME}/conf ${HOME_DIR}/${APACHE_VERSION}

cp -r ${AXISCPP_HOME}/deploy ${HOME_DIR}/${APACHE_VERSION}/Axis

replace "/usr/local" "${HOME_DIR}/${APACHE_VERSION}" < ${AXISCPP_HOME}/deploy/axiscpp.conf_linux > ${HOME_DIR}/${APACHE_VERSION}/Axis/axiscpp.conf 
replace "/usr/local" "${HOME_DIR}/${APACHE_VERSION}" < ${AXISCPP_HOME}/deploy/conf/server.wsdd_linux > ${HOME_DIR}/${APACHE_VERSION}/Axis/conf/server.wsdd 

AXIS_HOME=${HOME_DIR}/${APACHE_VERSION}/Axis
export AXIS_HOME

# *** Build Samples ***
LD_LIBRARY_PATH=${AXISCPP_HOME}/bin:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH

#cp ${HOME_DIR}/samples.server.configure.ac ${AXISCPP_HOME}/samples/server/configure.ac
#cp ${HOME_DIR}/samples.client.configure.ac ${AXISCPP_HOME}/samples/client/configure.ac

echo Starting Samples Build: Server Side
cd ${AXISCPP_HOME}/samples/server
echo Server samples build messages of build @ `date` > ${SAMPLES_BUILD_MESSAGES}
echo Server samples build errors/warnings of build @ `date` > ${SAMPLES_BUILD_ERRORS}

sh build.sh >> ${SAMPLES_BUILD_MESSAGES} 2>>${SAMPLES_BUILD_ERRORS}

if [ $? = 0 ]
then
	echo Server Samples Build  Sucessfull
	echo `date` Server Samples Build  Sucessfull >> ${LOG}
else
	echo Server Samples Build Failed
	echo `date` Server Samples Build Failed >> ${LOG}
	exit
fi

echo See the following log files for details
echo "	Summary log 				:${LOG}"
echo "	Server Samples Build Messages		:${SAMPLES_BUILD_MESSAGES}"
echo "	Server Samples Build errors/warnings 	:${SAMPLES_BUILD_ERRORS}"

echo Starting Samples Build: Client Side
cd ${AXISCPP_HOME}/samples/client
echo Client samples build messages of build @ `date` >> ${SAMPLES_BUILD_MESSAGES}
echo Client samples build errors/warnings of build @ `date` >> ${SAMPLES_BUILD_ERRORS}

sh build.sh >> ${SAMPLES_BUILD_MESSAGES} 2>>${SAMPLES_BUILD_ERRORS}

if [ $? = 0 ]
then
	echo Client Samples Build  Sucessfull
	echo `date` Client Samples Build  Sucessfull >> ${LOG}
else
	echo Client Samples Build Failed
	echo `date` Client Samples Build Failed >> ${LOG}
	exit
fi

echo See the following log files for details
echo "	Summary log 				:${LOG}"
echo "	Client Samples Build Messages		:${SAMPLES_BUILD_MESSAGES}"
echo "	Client Samples Build errors/warnings 	:${SAMPLES_BUILD_ERRORS}"

# *** Deploy with Apache 2 ***
echo Start deploy with ${APACHE_VERSION} 
rm -f ${HOME_DIR}/${APACHE_VERSION}/modules/libaxiscpp_mod2.so
rm -f ${AXIS_HOME}/libs/libserver_engine.so
rm -f ${AXIS_HOME}/webservices/libAdminService.so
cp -f ${AXISCPP_HOME}/bin/libAdminService.so ${AXIS_HOME}/webservices/
cp -f $AXISCPP_HOME/bin/libserver_engine.so ${AXIS_HOME}/libs/
cp -f $AXISCPP_HOME/bin/libaxiscpp_mod2.so ${HOME_DIR}/${APACHE_VERSION}/modules/


echo Startng apache 
cd ${HOME_DIR}
if [ ${HOME_DIR}/${APACHE_VERSION}/logs/httpd.pid ]
then
	kill `cat ${HOME_DIR}/${APACHE_VERSION}/logs/httpd.pid`
fi
${APACHE_HOME}/bin/httpd -k restart -f ${HOME_DIR}/${APACHE_VERSION}/httpd.conf

if [ $? = 0 ]
then
        echo Apache Start  Sucessfull
        echo `date` Apache Start  Sucessfull >> ${LOG}
else
        echo Apache Start Failed
        echo `date` Apache Start Failed >> ${LOG}
	exit
fi

echo Samples run messages @ `date` > ${SAMPLES_RUN_MESSAGES}
echo Samples run errors/warnings @ `date` > ${SAMPLES_RUN_ERRORS}

for sample in base cbase groupB cgroupB doclitbase doclitgroupB calculator; 
do
	ls | ${AXISCPP_HOME}/samples/client/${sample} localhost 9080 >> ${SAMPLES_RUN_MESSAGES} 2>>${SAMPLES_RUN_ERRORS}
	if [ $? = 0 ]
then
        echo ${sample} Sucessfull
        echo `date` ${sample} Sucessfull >> ${LOG}
else
        echo ${sample} Failed
        echo `date` ${sample} Failed >> ${LOG}
	exit
fi

done
